package pwc.taxtech.atms.service.impl;

import org.apache.commons.lang3.BooleanUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.Assert;
import pwc.taxtech.atms.common.*;
import pwc.taxtech.atms.common.message.LogMessage;
import pwc.taxtech.atms.common.message.OrganizationMessage;
import pwc.taxtech.atms.common.util.BeanUtil;
import pwc.taxtech.atms.constant.OrganizationConstant;
import pwc.taxtech.atms.dto.OperationLogDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.UpdateLogParams;
import pwc.taxtech.atms.dto.navtree.NavTreeDto;
import pwc.taxtech.atms.entity.ServiceType;
import pwc.taxtech.atms.organization.dao.OrganizationHKMapper;
import pwc.taxtech.atms.organization.dpo.OrganizationHKDto;
import pwc.taxtech.atms.organization.entity.OrganizationHK;
import pwc.taxtech.atms.organization.entity.OrganizationHKExample;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static pwc.taxtech.atms.common.CommonConstants.SystemError;
import static pwc.taxtech.atms.common.CommonUtils.copyProperties;
import static pwc.taxtech.atms.common.OperateLogType.OperationLogOrganization;
import static pwc.taxtech.atms.common.message.LogMessage.AddOrganization;
import static pwc.taxtech.atms.common.util.SpringContextUtil.distributedIdService;

@Service
public class OrganizationHKServiceImpl {
    private static final Logger logger = LoggerFactory.getLogger(OrganizationHKServiceImpl.class);

    @Resource
    private OrganizationHKMapper organizationHKMapper;

    @Autowired
    private OperationLogServiceImpl operationLogService;

    @Autowired
    private BeanUtil beanUtil;

    public List<OrganizationHK> getOrgInfo() {
        return organizationHKMapper.selectByExample(null);
    }

    public OperationResultDto enableOrgs(Long orgId) {
        OrganizationHK curOrg = orgId != null ? organizationHKMapper.selectByPrimaryKey(orgId) : null;
        if (curOrg == null) {
            return new OperationResultDto(false, OrganizationMessage.NotFindOrgEnable);
        } else {
            if (!curOrg.getIsActive()) {
                curOrg.setIsActive(true);
                organizationHKMapper.updateByPrimaryKey(curOrg);
                OperationLogDto opLog = new OperationLogDto();
                opLog.setAction(OperationAction.Update.value());
                opLog.setOperationContent(LogMessage.IsActive);
                opLog.setOperationObject(curOrg.getName());
                opLog.setOriginalState(LogMessage.Disable);
                opLog.setUpdateState(LogMessage.Enable);
                opLog.setIp("");
                opLog.setModule(OperationModule.Organization.value());
                opLog.setComment("");
                opLog.setLogType(OperateLogType.OperationLogOrganization.value());
                operationLogService.addOperationLog(opLog);
            }
            return new OperationResultDto(true);
        }
    }

    public OperationResultDto disableOrgs(List<Long> orgIds) {
        try {
            List<OperationLogDto> logList = new ArrayList<>();
            for (Long orgId : orgIds) {
                OrganizationHK rootOrg = orgId != null ? organizationHKMapper.selectByPrimaryKey(orgId) : null;
                Assert.notNull(rootOrg, "rootOrg is null");
                if (rootOrg.getIsActive()) {
                    rootOrg.setIsActive(false);
                    organizationHKMapper.updateByPrimaryKey(rootOrg);
                    OperationLogDto opLog = new OperationLogDto();
                    opLog.setAction(OperationAction.Update.value());
                    opLog.setOperationContent(LogMessage.IsActive);
                    opLog.setOperationObject(rootOrg.getName());
                    opLog.setOriginalState(BooleanUtils.isTrue(rootOrg.getIsActive()) ? "False" : "True");
                    opLog.setUpdateState(BooleanUtils.isTrue(rootOrg.getIsActive()) ? "True" : "False");
                    opLog.setModule(OperationModule.Organization.value());
                    opLog.setComment("");
                    opLog.setLogType(OperateLogType.OperationLogOrganization.value());
                    logList.add(opLog);
                }
            }
            operationLogService.addOperationLogList(logList);
            return new OperationResultDto(true);
        } catch (Exception ex) {
            logger.error("DisableOrgs出错了:" + ex, ex);
            logger.error("标记回滚, ready to call setRollbackOnly");
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new OperationResultDto(false, ex.getMessage());
        }
    }

    public OperationResultDto<Object> updateOrg(OrganizationHKDto orgDto) {
        try {
            Assert.notNull(orgDto, "orgDto Null Exception on updateOrg");
            OrganizationHK org = orgDto.getId() != null ? organizationHKMapper.selectByPrimaryKey(orgDto.getId()) : null;
            if (org == null) {
                return new OperationResultDto(false, OrganizationMessage.NotFindOrg);
            }
            OrganizationHK orgOriginal = new OrganizationHK();
            orgOriginal = CommonUtils.copyProperties(org, orgOriginal);
            // code唯一验证

            beanUtil.copyProperties(orgDto, org);
            org.setUpdateTime(new Date());
            OrganizationHK orgUpdate = org;
            organizationHKMapper.updateByPrimaryKeySelective(org);

            // 更新自定义属性
            UpdateLogParams updateLogParams = new UpdateLogParams();
            updateLogParams.setOriginalState(orgOriginal);
            updateLogParams.setUpdateState(orgUpdate);
            updateLogParams.setOperationModule(OperationModule.Organization.value());
            updateLogParams.setOperateLogType(OperationLogOrganization.value());
            updateLogParams.setOperationContent(orgUpdate.getName());
            updateLogParams.setOperationObject(org.getName());
            updateLogParams.setOperationAction(OperationAction.Update.value());
            updateLogParams.setComment("");
            operationLogService.updateDataAddLog(updateLogParams);
            return new OperationResultDto(true);
        } catch (DuplicateKeyException e) {
            //万不得已出此下策
            return new OperationResultDto(true, "保存成功!");
        } catch (Exception ex) {
            logger.error("UpdateOrg出错了:" + ex, ex);
            logger.error("标记回滚, ready to call setRollbackOnly");
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new OperationResultDto(false, ex.getMessage());
        }
    }

    public List<NavTreeDto> getOrgListToJson(Integer useType) {
        List<OrganizationHKDto> orgList = getOrgList(useType);
        return orgList.stream().map(this::rotateOrganizationDtoToNavTreeDto).collect(Collectors.toList());
    }

    public List<OrganizationHKDto> getOrgList(Integer useType) {
        List<OrganizationHKDto> orgDtoList = new ArrayList<>();
        List<OrganizationHK> orgList = findAllOrganizations();
        if (!orgList.isEmpty()) {
            // find rootOrg
            List<OrganizationHK> rootOrgs = null;
            if (useType == 1) {
                rootOrgs = orgList.stream().filter(x -> x.getParentId() == 0).collect(Collectors.toList());
            } else {
                rootOrgs = orgList.stream()
                        .filter(x -> x.getParentId() == 0 && CommonConstants.ACTIVE_STATUS.equals(x.getIsActive()))
                        .collect(Collectors.toList());
            }

            List<ServiceType> serviceList = new ArrayList<>();
            for (OrganizationHK rootOrg : rootOrgs) {
                OrganizationHKDto rootDto = rotateOrganizationToOrganizationDto(rootOrg);
                OrganizationHKDto subOrgs = genarateSubOrgs(rootDto, orgList, serviceList, useType);
                orgDtoList.add(subOrgs);
            }
        }
        orgDtoList.sort(Comparator.comparing(OrganizationHKDto::getName));
        return orgDtoList;
    }

    private NavTreeDto rotateOrganizationDtoToNavTreeDto(OrganizationHKDto organizationDto) {
        NavTreeDto navTreeDto = new NavTreeDto();
        navTreeDto.setLabel(organizationDto.getName());
        navTreeDto.setData(organizationDto);
        navTreeDto.setChildren(organizationDto.getSubOrgs().stream().map(this::rotateOrganizationDtoToNavTreeDto)
                .collect(Collectors.toList()));
        return navTreeDto;
    }

    private OrganizationHKDto rotateOrganizationToOrganizationDto(OrganizationHK organization) {
        OrganizationHKDto organizationDto = new OrganizationHKDto();
        CommonUtils.copyProperties(organization, organizationDto);
        return organizationDto;
    }

    private OrganizationHKDto genarateSubOrgs(OrganizationHKDto parentOrg, List<OrganizationHK> orgs,
                                              List<ServiceType> serviceList, Integer useType) {
        if (parentOrg.getParentId() > 0) {
            OrganizationHK organization = orgs.stream().filter(x -> x.getId().equals(parentOrg.getParentId()))
                    .collect(Collectors.toList()).get(0);
            parentOrg.setParentName(organization == null ? null : organization.getName());
        }
        List<OrganizationHK> subOrgs = null;
        if (useType == 1) {
            subOrgs = orgs.stream().filter(x -> parentOrg.getId().equals(x.getParentId())).collect(Collectors.toList());
        } else {
            subOrgs = orgs.stream().filter(x -> parentOrg.getId().equals(x.getParentId())
                    && CommonConstants.ACTIVE_STATUS.equals(x.getIsActive())).collect(Collectors.toList());
        }
        parentOrg.setSubOrgs(new ArrayList<>());

        // create and add subtrees to the current node
        for (OrganizationHK subOrg : subOrgs) {
            OrganizationHKDto subDto = rotateOrganizationToOrganizationDto(subOrg);
            subDto.setLevel(parentOrg.getLevel() == null ? 1 : parentOrg.getLevel() + 1);
            parentOrg.getSubOrgs().add(genarateSubOrgs(subDto, orgs, serviceList, useType));
        }
        return parentOrg;
    }

    public List<OrganizationHK> findAllOrganizations() {
        OrganizationHKExample organizationExample = new OrganizationHKExample();
        return organizationHKMapper.selectByExample(organizationExample);
    }

    public OrganizationHKDto getSingleOrgByOrgId(Long orgId) {
        OrganizationHK organizationHK = organizationHKMapper.selectByPrimaryKey(orgId);
        OrganizationHKDto result = new OrganizationHKDto();
        CommonUtils.copyProperties(organizationHK, result);
        Assert.notNull(result,
                "result is null");
        result.setLevel(result.getLevel() == null ? 0 : result.getLevel());
        if (result.getParentId() == 0) {
            result.setParentName("");
        } else {
            OrganizationHK tParent = organizationHKMapper.selectByExample(null).stream()
                    .filter(sa -> Objects.equals(sa.getId(), result.getParentId())).findFirst().orElse(null);
            Assert.notNull(tParent, "tParent is null");
            result.setParentName(tParent.getName());
        }
        return result;
    }

    public OperationResultDto<Object> addOrg(OrganizationHKDto orgDto) {
        try {
            OperationResultDto result = checkExist(orgDto);
            if (result != null && !BooleanUtils.isTrue(result.getResult())) {
                return new OperationResultDto(result.getResult(), result.getResultMsg());
            }
            if (Objects.equals(orgDto.getParentId(), OrganizationConstant.NoParentId)) {
                orgDto.setParentId(null);
                orgDto.setpLevel(0);
            } else {

            }

            orgDto.setId(distributedIdService.nextId());
            Date now = new Date();
            OrganizationHK org = copyProperties(orgDto, new OrganizationHK());
            org.setCreateTime(now);
            org.setUpdateTime(now);
            organizationHKMapper.insertSelective(org);

            UpdateLogParams tempCommonLogParms = new UpdateLogParams();
            tempCommonLogParms.setOperationModule(OperationModule.Organization.value());
            tempCommonLogParms.setComment(AddOrganization);
            tempCommonLogParms.setOperateLogType(OperationLogOrganization.value());
            tempCommonLogParms.setOperationContent(org.getName());
            tempCommonLogParms.setOperationAction(OperationAction.New.value());
            tempCommonLogParms.setOperationObject(orgDto.getName());
            tempCommonLogParms.setUpdateState("");
            tempCommonLogParms.setOriginalState("");
            operationLogService.addOrDeleteDataAddLog(tempCommonLogParms);
            return new OperationResultDto(true);
        } catch (Exception e) {
            logger.error("addOrg出错:" + e, e);
            logger.error("标记回滚, ready to call setRollbackOnly");
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new OperationResultDto(false, SystemError);
        }
    }

    private OperationResultDto checkExist(OrganizationHKDto orgDto) {
        Assert.notNull(orgDto, "orgDto null point error");
        OrganizationHK queryRoot = null;
            OrganizationHKExample example = new OrganizationHKExample();
            OrganizationHKExample.Criteria criteria = example.createCriteria();
            criteria.andCodeEqualTo(orgDto.getCode());
            if (orgDto.getId() != null) {
                criteria.andIdNotEqualTo(orgDto.getId());
            }
            queryRoot = organizationHKMapper.selectByExample(example).stream().findFirst().orElse(null);

        if (queryRoot != null && orgDto.getId() != null && orgDto.getId()==0) {
            return new OperationResultDto(false, OrganizationMessage.ClientCodeIsExist);
        }
        OperationResultDto operationResult = checkOrgNameOrOrgCode(orgDto);
        return operationResult;
    }

    private OperationResultDto checkOrgNameOrOrgCode(OrganizationHKDto orgDto) {
        OrganizationHKExample example = new OrganizationHKExample();
        if (orgDto.getId() == null) {
            example.or().andCodeEqualTo(orgDto.getCode()).andNameEqualTo(orgDto.getName());
        } else {
            example.or().andCodeEqualTo(orgDto.getCode()).andIdNotEqualTo(orgDto.getId())
                    .andNameEqualTo(orgDto.getName());
        }
        OrganizationHK query = organizationHKMapper.selectByExample(example).stream().findFirst().orElse(null);
        if (query != null) {
            return new OperationResultDto(false, OrganizationMessage.OrgNameOrCodeIsExiste);
        }
        return new OperationResultDto(true);
    }
}
